import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.Scanner;

public class arbeidsmedium implements Serializable {

	static final int TEMPERATURE = 0;
	static final int PRESSURE = 1;
	static final int HFG = 2;
	static final int TETTHET_LIQUID = 3;
	static final int TETTHET_VAPOR = 4;
	static final int VISKOSITET_LIQUID = 5; // dynamisk viskositet
	static final int VISKOSITET_VAPOR = 6; // dynamisk viskositet
	static final int K_LIQUID = 7;
	static final int K_VAPOR = 8;
	static final int SURFACETENSION_LIQUID = 9;
	static final int CP_LIQUID = 10;
	static final int CP_VAPOR = 11;

	public String arbeidsmediumNavn;
	public double molarMasse; // Mv

	int i;
	boolean solution_reached;

	public String[] arbeidsmediumKolonneOverskrifter = { "Temp [K]",
			"Metningstrykk [Kpa]", "Latent varme (hfg) [kJ/kg]",
			"Tetthet vske [kg/m^3]", "Tetthet damp [kg/m^3]",
			"Dynamisk Viskositet vske [N*s/m^2]",
			"Dynamisk Viskositet damp [N*s/m^2]",
			"Termisk konduktivitet vske [W/m*K]",
			"Termisk konduktivitet damp [W/m*K]",
			"Overflatespenning vske [N/m]", "Cp vske [kJ/kg*K]",
			"Cp damp [kJ/kg*K]" };

	public double[][] arbeidsmediumData = null;

	public double[][] lesFraFil(File arbeidsmediumfil) {

		int rad = 0;
		double[][] data = new double[10][12]; // datafil for arbeidsmedium m
		// ha max dette antall data
		// fordelt i [rader][kolonner]
		Scanner scanner = null;
		try {
			// Create a scanner to read the file
			scanner = new Scanner(arbeidsmediumfil);
		} catch (FileNotFoundException e) {
			System.out.println("Finner ikke arbeidsmediumfil "
					+ arbeidsmediumfil);
			// Stop program if no file found
			return null;
		}

		try {
			// first use a Scanner to get each line
			while (scanner.hasNextLine()) {
				// processLine( scanner.nextLine() );
				// use a second Scanner to parse the content of each line
				Scanner linjescanner = new Scanner(scanner.nextLine());
				linjescanner.useDelimiter(",");
				if (linjescanner.hasNext()) {

					for (int kolonne = 0; kolonne < 12; kolonne++) {

						try {
							data[rad][kolonne] = Double
									.parseDouble(linjescanner.next());
						} catch (NumberFormatException e) {
							System.out
									.println("Feil ved konvertering fra String til Double (arbeidsmedium)");
						}

					}

					// (no need for finally here, since String is source)
					linjescanner.close();
				} else {
					System.out
							.println("Empty or invalid line. Unable to process.");
				}
				rad++;
			}
		} finally {
			// ensure the underlying stream is always closed
			scanner.close();
		}

		return data;
	}

	// Read a value from a table of data corresponding to a input value and a
	// desired output value. Forutsetter at tabell er sortert i stigende eller
	// minkende verdier
	public double getProperty(double inputvalue, int input_type, int output_type) {// (*@\label{getProperty}@*)

		// er strre enn noe tall i tabell
		if (inputvalue > arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type]
				&& inputvalue > arbeidsmediumData[0][input_type]) // er strre
		// enn noe
		// tall i
		// tabell
		{
			if (arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type] > 
			arbeidsmediumData[(arbeidsmediumData.length - 2)][input_type])

			{
				return extrapolate(
						arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type],
						arbeidsmediumData[(arbeidsmediumData.length - 2)][input_type],
						arbeidsmediumData[(arbeidsmediumData.length - 1)][output_type],
						arbeidsmediumData[(arbeidsmediumData.length - 2)][output_type],
						inputvalue);
			} else {
				return extrapolate(arbeidsmediumData[1][input_type],
						arbeidsmediumData[0][input_type],
						arbeidsmediumData[1][output_type],
						arbeidsmediumData[0][output_type], inputvalue);
			}
		}

		// er mindre enn noe tall i tabell
		else if (inputvalue < arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type]
				&& inputvalue < arbeidsmediumData[0][input_type])

		{
			if (arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type] >
			arbeidsmediumData[(arbeidsmediumData.length - 2)][input_type])

			{
				return extrapolate(arbeidsmediumData[1][input_type],
						arbeidsmediumData[0][input_type],
						arbeidsmediumData[1][output_type],
						arbeidsmediumData[0][output_type], inputvalue);
			} else {
				return extrapolate(
						arbeidsmediumData[(arbeidsmediumData.length - 1)][input_type],
						arbeidsmediumData[(arbeidsmediumData.length - 2)][input_type],
						arbeidsmediumData[(arbeidsmediumData.length - 1)][output_type],
						arbeidsmediumData[(arbeidsmediumData.length - 2)][output_type],
						inputvalue);
			}
		}

		i = arbeidsmediumData.length / 2; // Start to look in the middle
		// of the array. (More effective
		// than to start in one end)
		solution_reached = false;

		while (!solution_reached) {
			if (inputvalue == arbeidsmediumData[i][input_type])

			{

				solution_reached = true;
				return arbeidsmediumData[i][output_type];
			} else if (inputvalue > arbeidsmediumData[i][input_type])

			{

				if (inputvalue < arbeidsmediumData[(i + 1)][input_type])

				{

					return interpolate(arbeidsmediumData[i][input_type],
							arbeidsmediumData[i + 1][input_type],
							arbeidsmediumData[i][output_type],
							arbeidsmediumData[i + 1][output_type], inputvalue);
				} else if (inputvalue < arbeidsmediumData[(i - 1)][input_type])

				{

					return interpolate(arbeidsmediumData[i - 1][input_type],
							arbeidsmediumData[i][input_type],
							arbeidsmediumData[i - 1][output_type],
							arbeidsmediumData[i][output_type], inputvalue);
				} else if (arbeidsmediumData[i][input_type] > arbeidsmediumData[i + 1][input_type])

				{
					i--;
				} else if (arbeidsmediumData[i][input_type] < arbeidsmediumData[i + 1][input_type])

				{
					i++;
				}
			}

			else if (inputvalue < arbeidsmediumData[i][input_type]) // Verdien

			{

				if (inputvalue > arbeidsmediumData[(i + 1)][input_type]) // ligger

				{

					return interpolate(arbeidsmediumData[i][input_type],
							arbeidsmediumData[i + 1][input_type],
							arbeidsmediumData[i][output_type],
							arbeidsmediumData[i + 1][output_type], inputvalue);
				} else if (inputvalue > arbeidsmediumData[(i - 1)][input_type]) // ligger

				{

					return interpolate(arbeidsmediumData[i - 1][input_type],
							arbeidsmediumData[i][input_type],
							arbeidsmediumData[i - 1][output_type],
							arbeidsmediumData[i][output_type], inputvalue);
				} else if (arbeidsmediumData[i][input_type] > arbeidsmediumData[i + 1][input_type])

				{
					i++;
				} else if (arbeidsmediumData[i][input_type] < arbeidsmediumData[i + 1][input_type])

				{
					i--;
				}
			} else {

				solution_reached = true;
			}

		}
		return 0.0;
	}

	private double interpolate(double lowerValueX, double upperValueX,
			double lowerValueY, double upperValueY, double middleValueX) {
		return upperValueY
				- ((upperValueX - middleValueX) * (upperValueY - lowerValueY) / (upperValueX - lowerValueX));
	}

	private double extrapolate(double lowerValueY, double middleValueY,
			double lowerValueX, double middleValueX, double upperValueY) {
		return ((middleValueX - lowerValueX) / (middleValueY - lowerValueY))
				* (upperValueY - lowerValueY) + lowerValueX;
	}

	public void skrivTilFil(File filnavn) {
		BufferedWriter bufferedWriter = null;

		try {

			bufferedWriter = new BufferedWriter(new FileWriter(filnavn));

			String linje = new String();

			for (int y = 0; y < arbeidsmediumData.length; y++) {
				for (int x = 0; x < 12; x++) {
					linje += Double.toString(arbeidsmediumData[y][x]);
					if (x < 11) {
						linje += ",";
					}
				}
				bufferedWriter.write(linje);
				linje = "";
				if (y < (arbeidsmediumData.length - 1)) {
					bufferedWriter.newLine();
				}
			}

		} catch (FileNotFoundException ex) {
			System.out.println("Kan ikke skrive til fil. Fant ikke fil");
		} catch (IOException ex) {
			System.out.println("IO feil");
		} finally {

			try {
				if (bufferedWriter != null) {
					bufferedWriter.flush();
					bufferedWriter.close();
				}
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}

	}
}
