import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.math.BigDecimal;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class prosjekt {
	varmeror varmeror;
	likningsett likningsett;
	chartFrame testframe;

	double surfacetension_water = 0.007564; // sigma

	// Begrensninger ved simulering av varmerrsbegrensninger
	double Qmin, Qmax, Tmin, Tmax;
	double capLimitKurve[], bolingLimitKurve[], sonicLimitKurve[],
			vicousLimitKurve[];

	double h_kondensasjonfilm = 0.0;
	double hDelta_kondensator = 0.0;
	double hDelta_fordamper = 0.0;

	public prosjekt() {
		varmeror = new varmeror();
		likningsett = new likningsett();
		Qmin = 0.0;
		Qmax = 0.0;
		Tmin = 0.0;
		Tmax = 0.0;
		// Dette er for testforml

	}

	// (*@\label{initDummyVariables}@*)
	public void initDummyVariables() {

		File file = new File("beregning1_vann.hpf");
		FileInputStream fis = null;
		ObjectInputStream in = null;
		try {
			fis = new FileInputStream(file);
			in = new ObjectInputStream(fis);
			varmeror = (varmeror) in.readObject();
			in.close();
			JOptionPane.showMessageDialog(null,
					"beregning1_vann.hpf er lastet inn i program!");
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null,
					"Feil ved pning av fil: beregning1_vann.hpf", "Feil",
					JOptionPane.ERROR_MESSAGE);
			ex.printStackTrace();

		} catch (ClassNotFoundException ex) {
			JOptionPane.showMessageDialog(null,
					"Feil ved pning av fil: beregning1_vann.hpf", "Feil",
					JOptionPane.ERROR_MESSAGE);
			ex.printStackTrace();
		}

	}

	public double[] ReLagkurve(double Qe, int steps, double temperatur_fordamper) {
		double[] dampkurve = lagDampkurve(Qe, steps, temperatur_fordamper);
		double[] ReKurve = new double[steps];

		double zincrease = (varmeror.geometri.lengdeFordamper
				+ varmeror.geometri.lengdeAdiabatisk + varmeror.geometri.lengdeKondensator)
				/ steps;
		double zvalue = zincrease;

		for (int z = 1; z < steps; z++) {
			// System.out.println("u_v "
			// +varmeror.arbeidsmedium.getProperty(punkter[z-1],
			// arbeidsmedium.PRESSURE, arbeidsmedium.VISKOSITET_VAPOR));
			ReKurve[z] = likningsett.Re_radiell(zvalue, varmeror.arbeidsmedium
					.getProperty(dampkurve[z], arbeidsmedium.PRESSURE,
							arbeidsmedium.VISKOSITET_VAPOR),
					varmeror.geometri.lengdeFordamper,
					varmeror.geometri.lengdeAdiabatisk,
					varmeror.geometri.lengdeKondensator, Qe,
					varmeror.arbeidsmedium.getProperty(dampkurve[z],
							arbeidsmedium.PRESSURE, arbeidsmedium.HFG));

			zvalue = zvalue + zincrease;
		}
		return ReKurve;
	}

	public chartFrame getRekurve(double punkter[], double stepsize,
			double temp, double fluks) {
		chartFrame dampkurve = new chartFrame(
				"Radiel Reynoldstall gjennom varmerr", "Lengde [m]",
				"Reynoldstall", punkter, "Reynoldstall", stepsize, // u0394=delta
				temp, fluks);
		return dampkurve;
	}

	public double[] lagDampkurve(double Qe, int steps, // (*@\label{lagDampkurve}@*)
			double temperatur_fordamper) {

		double[] punkter = new double[steps];
		double[] punkter_korrigert_differanse = new double[steps];

		double zincrease = (varmeror.geometri.lengdeFordamper
				+ varmeror.geometri.lengdeAdiabatisk + varmeror.geometri.lengdeKondensator)
				/ steps;
		double zvalue = zincrease;

		punkter[0] = varmeror.arbeidsmedium.getProperty(temperatur_fordamper,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.PRESSURE); // Initialverdi

		for (int z = 1; z < steps; z++) {
			// System.out.println("u_v "
			// +varmeror.arbeidsmedium.getProperty(punkter[z-1],
			// arbeidsmedium.PRESSURE, arbeidsmedium.VISKOSITET_VAPOR));
			punkter[z] = likningsett
					.Pv_by_simplified_closed_form_approximation_euler(
							punkter[z - 1], zincrease, zvalue,
							varmeror.arbeidsmedium.getProperty(punkter[z - 1],
									arbeidsmedium.PRESSURE,
									arbeidsmedium.VISKOSITET_VAPOR),
							varmeror.geometri.lengdeFordamper,
							varmeror.geometri.lengdeAdiabatisk,
							varmeror.geometri.lengdeKondensator, Qe,
							varmeror.arbeidsmedium.getProperty(punkter[z - 1],
									arbeidsmedium.PRESSURE, arbeidsmedium.HFG),
							varmeror.arbeidsmedium.getProperty(punkter[z - 1],
									arbeidsmedium.PRESSURE,
									arbeidsmedium.TETTHET_VAPOR),
							varmeror.geometri.indreRadius);
			// System.out.println("Punkt " + z + " = " + punkter[z]);
			zvalue = zvalue + zincrease;

			punkter_korrigert_differanse[z] = punkter[z] - punkter[0];
		}
		return punkter;
	}

	public chartFrame getDampkurve(double punkter[], double stepsize,
			double temp, double fluks) {
		chartFrame dampkurve = new chartFrame("Damptrykk gjennom varmerr",
				"Lengde [m]", "P [Pa]", punkter, "Damptrykk", stepsize, // u0394=delta
				temp, fluks);
		return dampkurve;
	}

	public double[] lagVeskekurve(double Qe, int steps,
			double temperatur_fordamper) {

		double[] punkter = new double[steps];
		double[] dampkurve = lagDampkurve(Qe, steps, temperatur_fordamper);

		double[] hydrostatiskTrykk = new double[steps];

		double[] reeltTrykk = new double[steps];

		double zincrease = (varmeror.geometri.lengdeFordamper
				+ varmeror.geometri.lengdeAdiabatisk + varmeror.geometri.lengdeKondensator)
				/ steps;
		double zvalue = zincrease;

		// lager frst en vanlig dampkurve med wetpoint i enden og sjekker om
		// dette er rett
		punkter[steps - 1] = dampkurve[steps - 1];

		for (int z = steps - 2; z >= 0; z--) { // (*@\label{lagVeskekurve_1st_loop}@*)

			punkter[z] = likningsett.Pl(punkter[z + 1], zincrease, zvalue,
					varmeror.arbeidsmedium.getProperty(punkter[z + 1],
							arbeidsmedium.PRESSURE,
							arbeidsmedium.VISKOSITET_LIQUID),
					varmeror.arbeidsmedium.getProperty(punkter[z + 1],
							arbeidsmedium.PRESSURE,
							arbeidsmedium.TETTHET_LIQUID), Math.PI
							* (Math.pow(varmeror.geometri.ytreRadius, 2) - Math
									.pow(varmeror.geometri.indreRadius, 2)),
					varmeror.veke.permeabilitet,
					varmeror.geometri.lengdeFordamper,
					varmeror.geometri.lengdeAdiabatisk,
					varmeror.geometri.lengdeKondensator, Qe,
					varmeror.arbeidsmedium.getProperty(punkter[z + 1],
							arbeidsmedium.PRESSURE, arbeidsmedium.HFG));

			hydrostatiskTrykk[z] = varmeror.arbeidsmedium.getProperty(
					punkter[z + 1], arbeidsmedium.PRESSURE,
					arbeidsmedium.TETTHET_LIQUID)
					* Math.sin(Math.toRadians(varmeror.geometri.tilt))
					* -9.81
					* zvalue;
			zvalue = zvalue + zincrease;

			reeltTrykk[z] = punkter[z] + hydrostatiskTrykk[z];
		}

		reeltTrykk[steps - 1] = punkter[steps - 1]
				+ hydrostatiskTrykk[steps - 1];

		int wetPointPosition = likningsett.getWetpoint(dampkurve, reeltTrykk,
				varmeror.geometri.lengdeKondensator,
				varmeror.geometri.lengdeAdiabatisk,
				varmeror.geometri.lengdeFordamper);

		if (wetPointPosition < (dampkurve.length - 1)) {
			punkter[wetPointPosition] = (dampkurve[dampkurve.length - 1] - 1000.0);
			double guess_starttrykk = dampkurve[dampkurve.length - 1];

			int endlessloopbrak = 0;

			while (Math.abs(reeltTrykk[wetPointPosition]
					- dampkurve[wetPointPosition]) > 0.0001) {
				zvalue = zincrease;

				guess_starttrykk += dampkurve[wetPointPosition]
						- reeltTrykk[wetPointPosition];

				punkter[steps - 1] = guess_starttrykk;

				for (int z = steps - 2; z >= 0; z--) {

					punkter[z] = likningsett.Pl(punkter[z + 1], zincrease,
							zvalue, varmeror.arbeidsmedium.getProperty(
									punkter[z + 1], arbeidsmedium.PRESSURE,
									arbeidsmedium.VISKOSITET_LIQUID),
							varmeror.arbeidsmedium.getProperty(punkter[z + 1],
									arbeidsmedium.PRESSURE,
									arbeidsmedium.TETTHET_LIQUID), Math.PI
									* (Math
											.pow(varmeror.geometri.ytreRadius,
													2) - Math.pow(
											varmeror.geometri.indreRadius, 2)),
							varmeror.veke.permeabilitet,
							varmeror.geometri.lengdeFordamper,
							varmeror.geometri.lengdeAdiabatisk,
							varmeror.geometri.lengdeKondensator, Qe,
							varmeror.arbeidsmedium.getProperty(punkter[z + 1],
									arbeidsmedium.PRESSURE, arbeidsmedium.HFG));

					hydrostatiskTrykk[z] = varmeror.arbeidsmedium.getProperty(
							punkter[z + 1], arbeidsmedium.PRESSURE,
							arbeidsmedium.TETTHET_LIQUID)
							* Math.sin(Math.toRadians(varmeror.geometri.tilt))
							* -9.81 * zvalue;
					zvalue = zvalue + zincrease;

					reeltTrykk[z] = punkter[z] + hydrostatiskTrykk[z];
				}

				endlessloopbrak++;
				if (endlessloopbrak > 10000) {
					System.out.println("Ikke gyldig lsning av vsketrykk");
					break;
				}
			}

		}
		reeltTrykk[steps - 1] = punkter[steps - 1]
				+ hydrostatiskTrykk[steps - 1];
		return reeltTrykk;
	}

	public chartFrame getVeskekurve(double punkter[], double stepsize,
			double temp, double fluks) {
		chartFrame veskekurve = new chartFrame("Vsketrykk gjennom varmerr",
				"Lengde [m]", "Trykk [Pa]", punkter, "Vsketrykk", stepsize,
				temp, fluks);
		return veskekurve;
	}

	public limitChartFrame getBegrensninger(double qmin, double qmax,
			double tmin, double tmax, double qStepSize, int EulerSteps) {
		capLimitKurve = new double[(int) (tmax - tmin)];
		bolingLimitKurve = new double[(int) (tmax - tmin)];
		sonicLimitKurve = new double[(int) (tmax - tmin)];
		vicousLimitKurve = new double[(int) (tmax - tmin)];

		double temp;
		boolean CapLimitFound;

		int t;
		double q;
		double veskeKurveIpunkt[];
		double dampkurveIpunkt[];

		for (t = 0; t < (int) (tmax - tmin); t++) { // (*@\label{getBegrensninger}@*)
			temp = tmin + t; // temp er temperaturen som skal brukes i
			// beregninger
			CapLimitFound = false;

			for (q = qmin; q <= qmax; q += qStepSize) {

				veskeKurveIpunkt = lagVeskekurve(q, EulerSteps, temp);
				dampkurveIpunkt = lagDampkurve(q, EulerSteps, temp);

				if (veskeKurveIpunkt[0] < 0
						|| dampkurveIpunkt[likningsett
								.getLowestValuePosition(dampkurveIpunkt)] < 0) // (*@\label{viscosLimit}@*)
				{
					vicousLimitKurve[t] = q;
					CapLimitFound = true;
					q = qmax;
				}

				else if (likningsett.capilary_limit_reached(
						varmeror.arbeidsmedium.getProperty(temp,
								arbeidsmedium.TEMPERATURE,
								arbeidsmedium.SURFACETENSION_LIQUID),
						varmeror.veke.effektivPoreradius, lagDampkurve(q,
								EulerSteps, temp), veskeKurveIpunkt,
						varmeror.arbeidsmedium.getProperty(temp,
								arbeidsmedium.TEMPERATURE,
								arbeidsmedium.TETTHET_LIQUID),
						varmeror.geometri.tilt,
						varmeror.geometri.lengdeFordamper,
						varmeror.geometri.lengdeAdiabatisk,
						varmeror.geometri.lengdeKondensator)
						&& !CapLimitFound) {
					CapLimitFound = true;
					capLimitKurve[t] = q;
					q = qmax;
				}

			}

			// gir egentlig ut i Watt s m derfor gange med 0.001 for  f
			// Kilowatt (hfg er gitt i KJ/KG)
			bolingLimitKurve[t] = 0.001 * likningsett.boilingLimit(
					varmeror.geometri.lengdeFordamper,
					varmeror.veke.konduktivitet,
					varmeror.geometri.ytreRadius, // radius av innervegg
					varmeror.geometri.indreRadius,// vapor space radius
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.SURFACETENSION_LIQUID),
					temp,
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE, arbeidsmedium.HFG),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.TETTHET_VAPOR),
					varmeror.veke.effektivPoreradius, // Rmem kan settes til
					// effektiv poreradius
					// iflge faghri
					temp, varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE, arbeidsmedium.K_LIQUID),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.VISKOSITET_VAPOR),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.VISKOSITET_LIQUID), q);

			sonicLimitKurve[t] = 0.001 * likningsett
					.sonicLimit(
							varmeror.geometri.indreRadius,
							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.HFG),
							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.TETTHET_VAPOR),

							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.CP_VAPOR)
									/ (varmeror.arbeidsmedium.getProperty(temp,
											arbeidsmedium.TEMPERATURE,
											arbeidsmedium.CP_VAPOR) - 8.314 / varmeror.arbeidsmedium.molarMasse),
							temp);

		}
		limitChartFrame graf = new limitChartFrame(capLimitKurve,
				bolingLimitKurve, sonicLimitKurve, vicousLimitKurve, tmin);
		return graf;
	}

	public String getCapLimit(double Qe, int steps, double temperatur_fordamper) {
		String contentstring = "Kalkulerer maksimum kappilrtrykk for varmerr.<BR>Fluks i fordamper Qe:"
				+ Double.toString(Qe)
				+ " kW<BR> Damptemperatur i fordamper Te:"
				+ Double.toString(temperatur_fordamper) + "&#176;K<BR><BR>";
		double[] dampkurve = lagDampkurve(Qe, steps, temperatur_fordamper);
		double[] veskekurve = lagVeskekurve(Qe, steps, temperatur_fordamper);

		double middeltrykk = (veskekurve[steps - 1] + veskekurve[0]) * 0.5;

		contentstring += "Hgeste vsketrykk i kondensator: "
				+ Double.toString(veskekurve[steps - 1]) + " Pa<BR>";
		contentstring += "Laveste vsketrykk i fordamper: "
				+ Double.toString(veskekurve[0]) + " Pa<BR>";
		contentstring += "Vskeegenskaper vurderes ved middeltrykk: "
				+ Double.toString(middeltrykk) + " Pa<BR><BR>";

		double sigma = varmeror.arbeidsmedium.getProperty(middeltrykk,
				arbeidsmedium.PRESSURE, arbeidsmedium.SURFACETENSION_LIQUID);
		double rho = varmeror.arbeidsmedium.getProperty(middeltrykk,
				arbeidsmedium.PRESSURE, arbeidsmedium.TETTHET_LIQUID);
		contentstring += "Overfaltespenning &#963; =" + Double.toString(sigma)
				+ " N/m<BR>";
		contentstring += "Tetthet &#961; =" + Double.toString(rho)
				+ " kg/m^3<BR><BR>";

		double pkappmax = likningsett.Pkapp_max(sigma,
				varmeror.veke.effektivPoreradius);
		double totaltTrykktap = Math
				.abs((veskekurve[0] - veskekurve[veskekurve.length - 1])
						+ Math.abs(dampkurve[0]
								- dampkurve[dampkurve.length - 1]))
				+ rho
				* 9.81
				* Math.sin(Math.toRadians(varmeror.geometri.tilt))
				* (varmeror.geometri.lengdeFordamper
						+ varmeror.geometri.lengdeAdiabatisk + varmeror.geometri.lengdeKondensator);
		if (likningsett.capilary_limit_reached(sigma,
				varmeror.veke.effektivPoreradius, dampkurve, veskekurve, rho,
				varmeror.geometri.tilt, varmeror.geometri.lengdeFordamper,
				varmeror.geometri.lengdeAdiabatisk,
				varmeror.geometri.lengdeKondensator)) {
			contentstring += "<FONT COLOR=red><B>ADVARSEL:</B> Kappilrbegrensning ndd<BR>"
					+ "Maksimum kappiltrykk er "
					+ Double.toString(pkappmax)
					+ " Pa<BR>"
					+ "Totalt trykkfall er "
					+ Double.toString(totaltTrykktap) + " Pa</FONT><BR>	";
		}

		else {
			contentstring += "<FONT COLOR=blue>Kappilrbegrensning er ikke ndd<BR>"
					+ "Maksimum kappiltrykk er "
					+ Double.toString(pkappmax)
					+ " Pa<BR>"
					+ "Totalt trykkfall er "
					+ Double.toString(totaltTrykktap) + " Pa</FONT><BR>	";
		}
		return contentstring;
	}

	public String gethDelta_kondensator(double Qe, int steps,
			double TsatFordamper) // termisk motstand i damp-vske grensesnitt
	{
		double[] dampkurve = lagDampkurve(Qe, steps, TsatFordamper);
		double[] veskekurve = lagVeskekurve(Qe, steps, TsatFordamper);

		double alpha = 1.0; // koeffisient foresltt av Faghri til  sette til 1
		// . annet for forurensede medier

		int kondensatorInngangPosisjon = (int) (steps - varmeror.geometri.lengdeKondensator
				/ ((varmeror.geometri.lengdeAdiabatisk
						+ varmeror.geometri.lengdeFordamper + varmeror.geometri.lengdeKondensator) / steps));

		double kondensatorInngangTemp = varmeror.arbeidsmedium.getProperty(
				dampkurve[kondensatorInngangPosisjon], arbeidsmedium.PRESSURE,
				arbeidsmedium.TEMPERATURE);
		double kondensatorUtgangTemp = varmeror.arbeidsmedium.getProperty(
				dampkurve[dampkurve.length - 1], arbeidsmedium.PRESSURE,
				arbeidsmedium.TEMPERATURE);
		double Tv_middel = likningsett.aritmetiskMiddelTemperatur(
				kondensatorInngangTemp, kondensatorUtgangTemp);

		double kondensatorInngangVeskeTemp = varmeror.arbeidsmedium
				.getProperty(veskekurve[kondensatorInngangPosisjon],
						arbeidsmedium.PRESSURE, arbeidsmedium.TEMPERATURE);
		double kondensatorUtgangVeskeTemp = varmeror.arbeidsmedium.getProperty(
				veskekurve[dampkurve.length - 1], arbeidsmedium.PRESSURE,
				arbeidsmedium.TEMPERATURE);
		double Tl_middel = likningsett.aritmetiskMiddelTemperatur(
				kondensatorInngangVeskeTemp, kondensatorUtgangVeskeTemp);

		double TsatKondenser = Tv_middel;

		double hfg = varmeror.arbeidsmedium.getProperty(TsatKondenser,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.HFG);
		double v_damp = 1 / varmeror.arbeidsmedium.getProperty(TsatKondenser,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.TETTHET_VAPOR);
		double v_veske = 1 / varmeror.arbeidsmedium.getProperty(TsatKondenser,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.VISKOSITET_LIQUID);
		double vfg = v_veske - v_damp;
		double rho_v = 1 / v_damp;

		double Mv = varmeror.arbeidsmedium.molarMasse;

		double h = likningsett.h_delta(alpha, hfg, Tv_middel, vfg, rho_v, Mv);
		String contentstring = "<TR><TD>Damptemperatur ved begynnelse av kondensator:</TD><TD>"
				+ Double.toString(round(kondensatorInngangTemp, 2))
				+ "</TD><TD>&#176;K</TD></TR>"
				+ "<TR><TD>Damptemperatur ved ende av kondensator:</TD><TD>"
				+ Double.toString(round(kondensatorUtgangTemp, 2))
				+ "</TD><TD>&#176;K</TD></TR>"
				+ "<TR><TD>Aritmetisk damp temperatur i kondensator (Tv): </TD><TD>"
				+ Double.toString(round(Tv_middel, 2))
				+ "</TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Aritmetisk vesketemperatur i kondensator (Tl):</TD><TD> "
				+ Double.toString(round(Tl_middel, 2))
				+ "</TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Varmetransportkoeffisient damp-vskegrensesnitt i kondensator h&#948;</TD><TD><FONT COLOR=green>"
				+ Double.toString(round(h, 2))
				+ "</FONT></TD><TD>W/m^2*K</TD></TR>";
		hDelta_kondensator = h;
		return contentstring;
	}

	public String gethDelta_fordamper(double Qe, int steps, double TsatFordamper) {
		double[] dampkurve = lagDampkurve(Qe, steps, TsatFordamper);
		double[] veskekurve = lagVeskekurve(Qe, steps, TsatFordamper);

		double alpha = 1.0; // koeffisient foresltt av Faghri til  sette til 1
		// . annet for forurensede medier

		int fordamperUtgangPosisjon = (int) (varmeror.geometri.lengdeFordamper / ((varmeror.geometri.lengdeAdiabatisk
				+ varmeror.geometri.lengdeFordamper + varmeror.geometri.lengdeKondensator) / steps));

		double fordamperDampInngangTemp = varmeror.arbeidsmedium
				.getProperty(dampkurve[0], arbeidsmedium.PRESSURE,
						arbeidsmedium.TEMPERATURE);
		double fordamperDampUtgangTemp = varmeror.arbeidsmedium.getProperty(
				dampkurve[fordamperUtgangPosisjon], arbeidsmedium.PRESSURE,
				arbeidsmedium.TEMPERATURE);

		double Tv_middel = likningsett.aritmetiskMiddelTemperatur(
				fordamperDampInngangTemp, fordamperDampUtgangTemp);

		double fordamperDampInngangVeskeTemp = varmeror.arbeidsmedium
				.getProperty(veskekurve[0], arbeidsmedium.PRESSURE,
						arbeidsmedium.TEMPERATURE);
		double fordamperDampUtgangVeskeTemp = varmeror.arbeidsmedium
				.getProperty(veskekurve[fordamperUtgangPosisjon],
						arbeidsmedium.PRESSURE, arbeidsmedium.TEMPERATURE);

		double Tl_middel = likningsett.aritmetiskMiddelTemperatur(
				fordamperDampInngangVeskeTemp, fordamperDampUtgangVeskeTemp);

		double hfg = varmeror.arbeidsmedium.getProperty(Tv_middel,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.HFG);
		double v_damp = 1 / varmeror.arbeidsmedium.getProperty(Tv_middel,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.TETTHET_VAPOR);
		double v_veske = 1 / varmeror.arbeidsmedium.getProperty(Tv_middel,
				arbeidsmedium.TEMPERATURE, arbeidsmedium.VISKOSITET_LIQUID);
		double vfg = v_veske - v_damp;
		double rho_v = 1 / v_damp;

		double Mv = varmeror.arbeidsmedium.molarMasse;

		// Tester om det er ok  neglisjere kompressibillitet (anntar mettet
		// damp ut av fordamper)
		String contentstring = "";
		double Pv = dampkurve[fordamperUtgangPosisjon];
		if (!likningsett.okToNegclectCompressibleFlow(
				// (*@\label{ytelse5}@*)
				varmeror.geometri.lengdeFordamper,
				varmeror.geometri.lengdeFordamper,
				varmeror.geometri.lengdeAdiabatisk,
				varmeror.geometri.lengdeKondensator, Qe, varmeror.arbeidsmedium
						.getProperty(Pv, arbeidsmedium.PRESSURE,
								arbeidsmedium.HFG), varmeror.arbeidsmedium
						.getProperty(Pv, arbeidsmedium.PRESSURE,
								arbeidsmedium.TETTHET_VAPOR), Pv,
				varmeror.arbeidsmedium.getProperty(Pv, arbeidsmedium.PRESSURE,
						arbeidsmedium.CP_VAPOR), varmeror.arbeidsmedium
						.getProperty(Pv, arbeidsmedium.PRESSURE,
								arbeidsmedium.CP_VAPOR) / 1.6, Math.PI
						* Math.pow(varmeror.geometri.indreRadius, 2)))
			contentstring += "<TR><TD colspan=3><FONT color=red><B>Advarsel:</B> Neglisjerer  kompressibilitet til tross for Ma > 0 .3</font></TD></TR>";

		double h = likningsett.h_delta(alpha, hfg, Tv_middel, vfg, rho_v, Mv);

		contentstring += "<TR><TD>Damptemperatur ved begynnelse av fordamper:</TD><TD>"
				+ Double.toString(round(fordamperDampInngangTemp, 2))
				+ "</TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Damptemperatur ved fordamperutgang:</TD><TD>"
				+ Double.toString(round(fordamperDampUtgangTemp, 2))
				+ "</TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Aritmetisk damp temperatur i fordamper (Tv): </TD><TD>"
				+ Double.toString(round(Tv_middel, 2))
				+ "</TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Aritmetisk vesketemperatur i fordamper (Tl):</TD><TD> "
				+ Double.toString(round(Tl_middel, 2))
				+ "</TD><TD> &#176;K</TD></TR>"

				+ "<TR><TD>Varmetransportkoeffisient damp-vskegrensesnitt i fordamper h&#948;:</TD><TD> <FONT COLOR=green>"
				+ Double.toString(round(h, 2))
				+ "</FONT></TD><TD>  W/m^2*K</TD></TR>";
		hDelta_fordamper = h;

		return contentstring;
	}

	public String getOutsideTempeKondensator(double Qe, int steps,
			double TfordamperUtside) {
		double Tv_guess = TfordamperUtside - 1; // (*@\label{ytelse1}@*)
		double tolerance = 0.00000001;
		double TfordamperUtside_Beregnet = TfordamperUtside + 1;
		double keff = 0.0;
		double Tkondensator_Beregnest;
		String contentstring2 = "";
		String contentstring = "<TABLE BORDER=0 CELLPADDING=3><TR><TD Width=100%><B>Variabel</B></TD><TD><B>Verdi</B></TD><TD><B>Enhet</B></TD></TR>";
		int endlessloopwarning = 0;
		double R_fordamper = 0;

		while (Math.abs(TfordamperUtside - TfordamperUtside_Beregnet) > tolerance) {
			contentstring2 = gethDelta_fordamper(Qe, steps, Tv_guess);// (*@\label{ytelse2}@*)

			double kl = varmeror.arbeidsmedium.getProperty(Tv_guess,
					arbeidsmedium.TEMPERATURE, arbeidsmedium.K_LIQUID);
			keff = likningsett.keff(varmeror.veke.vekeType, kl,
					varmeror.veke.konduktivitet, varmeror.veke.porositet);

			R_fordamper = likningsett.R_fordamper(hDelta_fordamper, keff,
					varmeror.geometri.beholderKonduktivitet,
					varmeror.geometri.indreRadius,
					varmeror.geometri.ytreRadius, varmeror.geometri.ytreRadius
							+ varmeror.geometri.beholderTykkelse,
					varmeror.geometri.lengdeFordamper);
			TfordamperUtside_Beregnet = Tv_guess + Qe * R_fordamper;// (*@\label{ytelse3}@*)

			if (Math.abs(TfordamperUtside - TfordamperUtside_Beregnet) > tolerance)
				Tv_guess += TfordamperUtside - TfordamperUtside_Beregnet;// (*@\label{ytelse4}@*)
			endlessloopwarning++;
			if (endlessloopwarning > 1000)
				break;
		}
		contentstring += contentstring2;
		contentstring += "<TR><TD>Fant lsning etter " + endlessloopwarning
				+ " itterasjoner</TD></TR>";
		contentstring += "<TR><TD>Ty ytterside fordamper beregnet</TD><TD>"
				+ Double.toString(round(TfordamperUtside_Beregnet, 2))
				+ "</TD><TD> &#176;K</TD></TR>";

		contentstring += gethDelta_kondensator(Qe, steps, Tv_guess);
		double R_kondensator = likningsett.R_kondensator(hDelta_kondensator,
				1.0, keff, varmeror.geometri.beholderKonduktivitet,
				varmeror.geometri.indreRadius, varmeror.geometri.ytreRadius,
				varmeror.geometri.ytreRadius
						+ varmeror.geometri.beholderTykkelse,
				varmeror.geometri.lengdeKondensator);
		Tkondensator_Beregnest = Tv_guess - Qe * R_kondensator;
		contentstring += "<TR><TD>Ty ytterside kondensator beregnet til</TD><TD><FONT Color=green>"
				+ Double.toString(round(Tkondensator_Beregnest, 2))
				+ "</FONT></TD><TD> &#176;K</TD></TR>"
				+ "<TR><TD>Varmemotstand i fordamper (R_fordamper):</TD><TD><FONT Color=green>"
				+ Double.toString(round(R_fordamper, 2))
				+ "</FONT></TD><TD> &#176;K/W</TD></TR>"
				+ "<TR><TD>Varmemotstand i kondensator (R_kondensator):</TD><TD><FONT Color=green>"
				+ Double.toString(round(R_kondensator, 2))
				+ "</FONT></TD><TD> &#176;K/W</TD></TR>"

				+ "<TR><TD>Temperaturdifferanse &Delta;T over varmerr</TD><TD><FONT Color=green>"
				+ Double.toString(round(TfordamperUtside
						- Tkondensator_Beregnest, 2))
				+ "</FONT></TD><TD> &#176;K</TD></TR>";

		double varmeledningsevne = (Qe * 1000
				* varmeror.geometri.lengdeAdiabatisk
				+ varmeror.geometri.lengdeFordamper + varmeror.geometri.lengdeKondensator)
				/ (Math.PI
						* Math.pow(varmeror.geometri.ytreRadius
								+ varmeror.geometri.beholderTykkelse, 2) * (TfordamperUtside - Tkondensator_Beregnest));
		contentstring += "<TR><TD>Termisk varmeledningsevne</TD><TD><FONT Color=green>"
				+ Double.toString(round(varmeledningsevne, 2))
				+ "</FONT></TD><TD> W/m*&#176;K</TD></TR>";

		contentstring += "</TABLE>";

		return contentstring;
	}

	public String minimumDiameter(double qe, int steps, double temp)// (*@\label{minimumDiameter}@*)
	{
		String contentstring = "<TABLE BORDER=0 CELLPADDING=3><TR><TD Width=100%><B>Variabel</B></TD><TD><B>Verdi</B></TD><TD><B>Enhet</B></TD></TR>";
		double Rv = varmeror.geometri.indreRadius;
		double Ry = varmeror.geometri.ytreRadius;
		double L = varmeror.geometri.beholderTykkelse;

		double step = 0.00001;

		double[] veskeKurveIpunkt;
		double[] dampkurveIpunkt;
		Double Qe = 0.0;
		boolean LimitNotReached = true;

		while (LimitNotReached) {
			Qe = qe * (varmeror.geometri.ytreRadius + L) * 2 * Math.PI
					* varmeror.geometri.lengdeFordamper;

			veskeKurveIpunkt = lagVeskekurve(Qe, steps, temp);
			dampkurveIpunkt = lagDampkurve(Qe, steps, temp);
			if (veskeKurveIpunkt[0] < 0
					|| dampkurveIpunkt[likningsett
							.getLowestValuePosition(dampkurveIpunkt)] < 0) {
				contentstring += "<TR><TD>Vesketrykket i fordamper er mindre enn 0, Rv</TD><TD><FONT Color=green>"
						+ Double.toString(varmeror.geometri.indreRadius)
						+ "</TD></TR>";
				LimitNotReached = false;
			}

			else if (likningsett.capilary_limit_reached(varmeror.arbeidsmedium
					.getProperty(temp, arbeidsmedium.TEMPERATURE,
							arbeidsmedium.SURFACETENSION_LIQUID),
					varmeror.veke.effektivPoreradius, lagDampkurve(Qe, steps,
							temp), veskeKurveIpunkt, varmeror.arbeidsmedium
							.getProperty(temp, arbeidsmedium.TEMPERATURE,
									arbeidsmedium.TETTHET_LIQUID),
					varmeror.geometri.tilt, varmeror.geometri.lengdeFordamper,
					varmeror.geometri.lengdeAdiabatisk,
					varmeror.geometri.lengdeKondensator)) {
				LimitNotReached = false;
				contentstring += "<TR><TD>Kapillrbegrensning funnet ved Rv</TD><TD><FONT Color=green>"
						+ Double.toString(varmeror.geometri.indreRadius)
						+ "</TD></TR>";
			}

			double boiling = 0.001 * likningsett.boilingLimit(
					varmeror.geometri.lengdeFordamper,
					varmeror.veke.konduktivitet,
					varmeror.geometri.ytreRadius, // radius av innervegg
					varmeror.geometri.indreRadius,// vapor space radius
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.SURFACETENSION_LIQUID),
					temp,
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE, arbeidsmedium.HFG),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.TETTHET_VAPOR),
					varmeror.veke.effektivPoreradius, // Rmem kan settes til
					// effektiv poreradius
					// iflge faghri
					temp, varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE, arbeidsmedium.K_LIQUID),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.VISKOSITET_VAPOR),
					varmeror.arbeidsmedium.getProperty(temp,
							arbeidsmedium.TEMPERATURE,
							arbeidsmedium.VISKOSITET_LIQUID), Qe);

			if (boiling < Qe) {
				LimitNotReached = false;
				contentstring += "<TR><TD>kokebegrensning ved Rv</TD><TD><FONT Color=green>"
						+ Double.toString(varmeror.geometri.indreRadius)
						+ "</TD><TD>m</TD></TR>";
			}

			double sonic = 0.001 * likningsett
					.sonicLimit(
							varmeror.geometri.indreRadius,
							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.HFG),
							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.TETTHET_VAPOR),
							varmeror.arbeidsmedium.getProperty(temp,
									arbeidsmedium.TEMPERATURE,
									arbeidsmedium.CP_VAPOR)
									/ (varmeror.arbeidsmedium.getProperty(temp,
											arbeidsmedium.TEMPERATURE,
											arbeidsmedium.CP_VAPOR) - 8.314 / varmeror.arbeidsmedium.molarMasse),
							temp);
			if (sonic < Qe) {
				LimitNotReached = false;
				contentstring += "<TR><TD>Sonisk begrensning ved Rv</TD><TD><FONT Color=green>"
						+ Double.toString(varmeror.geometri.indreRadius)
						+ "</TD><TD>m</TD></TR>";
			}

			varmeror.geometri.indreRadius -= step;
			varmeror.geometri.ytreRadius -= step;
			if (varmeror.geometri.indreRadius < 0) {
				contentstring += "<TR><TD>Rv mindre enn null</TD><TD><FONT Color=green>+"
						+ Double.toString(varmeror.geometri.indreRadius)
						+ "</TD></TR>";
				break;
			}
		}

		varmeror.geometri.indreRadius = Rv;
		varmeror.geometri.ytreRadius = Ry;
		contentstring += "<TR><TD>Varmebelastning Qe</TD><TD><FONT Color=green>"
				+ Double.toString(Qe) + " </TD><TD>kW</TD></TR>";
		contentstring += "<TR><TD>Spesifikk varmefluks</TD><TD><FONT Color=green>"
				+ Double.toString(qe) + "</TD><TD>kW/m^2</TD></TR>";
		contentstring += "<TR><TD>Damptemperatur ved fordamperstart</TD><TD><FONT Color=green>"
				+ Double.toString(temp) + "</TD><TD>K</TD></TR>";
		contentstring += "</TABLE>";
		return contentstring;
	}

	public static double round(double d, int decimalPlace) {
		BigDecimal bd = new BigDecimal(Double.toString(d));
		bd = bd.setScale(decimalPlace, BigDecimal.ROUND_HALF_UP);
		return bd.doubleValue();
	}

}// EOC

